/*
 * 
 * P2P anonymous BBS system zigumo
 * website:http://www.zigumo.org/
 * e-mail:hoge@zigumo.org
 *
 * Copyright (C) 2002 hoge-taicho
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * $Id: PushMessage.cs,v 1.4 2003/04/24 18:13:04 hoge-taicho Exp $
 * 
 */

using System;
using System.Net;

namespace Zigumo.MessageObject
{
	/// <summary>
	/// pushNX
	/// 0-15	16	bZ[WIDiGuidj
	/// 16		 1	Description(int)
	/// 17		 1	TTL(int)
	/// 18		 1	Hops(int)
	/// 19-22	 4	yC[hTCY(int)
	/// ------------------------------------
	/// 23-24	 2	|[gԍ(int)
	/// 25-28	 4	IPAhX(IPAddress)
	/// 29-44	16	T[ogID(Guid)
	/// 45-60	16	XbhID(Guid)
	/// 61-62	 2	JnXNo(int)
	/// 63-64	 2	IXNo(int)
	/// 65-end	 x	JeS(byte[])
	/// </summary>
	public class PushMessage : Zigumo.MessageObject.Message
	{
		/// <summary>
		/// ϒGAłJeS܂܂Ȃꍇ̃yC[hTCY
		/// </summary>
		public const int basePayloadSize = 42;

		public PushMessage(int length) : base(length) {
		}

		/// <summary>
		/// |[gԍ̃vpeB
		/// </summary>
		public int Port {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return (int)IPAddress.NetworkToHostOrder(BitConverter.ToInt16(data, 23));
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				BitConverter.GetBytes(IPAddress.HostToNetworkOrder((Int16)value)).CopyTo(data, 23);
			}
		}
		/// <summary>
		/// IPAhX̃vpeB
		/// </summary>
		public IPAddress Address {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return new IPAddress(BitConverter.ToInt32(data, 25));
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				((IPAddress)value).GetAddressBytes().CopyTo(data, 25);
			}
		}
		/// <summary>
		/// T[ogID̃vpeB
		/// </summary>
		public byte[] ServentID {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				byte[] id = new byte[16];
				Array.Copy(data, 29, id, 0, 16);
				return id;
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();
			
				Array.Copy(((byte[])value), 0, data, 29, 16);
			}
		}
		/// <summary>
		/// T[ogIDGUIDNXƂĈvpeB
		/// </summary>
		public Guid ServentIDToGuid {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				byte[] guid = new byte[16];
				Array.Copy(data, 29, guid, 0, 16);
				return new Guid(guid);
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				byte[] guid = ((Guid)value).ToByteArray();
				guid.CopyTo(data, 29);
			}
		}
		/// <summary>
		/// XbhID̃vpeB
		/// </summary>
		public byte[] ThreadID {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				byte[] id = new byte[16];
				Array.Copy(data, 45, id, 0, 16);
				return id;
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();
			
				Array.Copy(((byte[])value), 0, data, 45, 16);
			}
		}
		/// <summary>
		/// XbhIDGUIDNXƂĈvpeB
		/// </summary>
		public Guid ThreadIDToGuid {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				byte[] guid = new byte[16];
				Array.Copy(data, 45, guid, 0, 16);
				return new Guid(guid);
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				byte[] guid = ((Guid)value).ToByteArray();
				guid.CopyTo(data, 45);
			}
		}
		/// <summary>
		/// XNõvpeB
		/// </summary>
		public int StartResNo {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return (int)IPAddress.NetworkToHostOrder(BitConverter.ToInt16(data, 61));
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				BitConverter.GetBytes(IPAddress.HostToNetworkOrder((Int16)value)).CopyTo(data, 61);
			}
		}
		/// <summary>
		/// XNõvpeB
		/// </summary>
		public int EndResNo {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return (int)IPAddress.NetworkToHostOrder(BitConverter.ToInt16(data, 63));
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				BitConverter.GetBytes(IPAddress.HostToNetworkOrder((Int16)value)).CopyTo(data, 63);
			}
		}
		/// <summary>
		/// JeS̃vpeB
		/// </summary>
		public byte[] Category {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				byte[] category = new byte[(this.PayloadSize - PushMessage.basePayloadSize)];
				Array.Copy(data, 65, category, 0, this.PayloadSize - PushMessage.basePayloadSize);
				return category;
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				Array.Copy(((byte[])value), 0, data, 65, ((byte[])value).Length);
			}
		}
		/// <summary>
		/// wb_[\bh
		/// </summary>
		public override void CreateHeader() {
			base.CreateHeader();
			this.Type = (byte)MessageType.Push;
			this.PayloadSize = 0;
		}

		/// <summary>
		/// bZ[W̖OԂ\bh
		/// </summary>
		/// <returns>bZ[W</returns>
		public override string ToString() {
			return "Push";
		}	
	}
}
